@@ -1,71 +1,76 @@
+from typing import Any, Dict, List, Optional, Union
+
 from tableau_api_lib.api_endpoints import BaseEndpoint
 
 
 class WorkbookEndpoint(BaseEndpoint):
     def __init__(self,
                  ts_connection,
-                 query_workbooks=False,
-                 query_workbook=False,
-                 publish_workbook=False,
-                 update_workbook=False,
-                 delete_workbook=False,
-                 update_workbook_connection=False,
-                 workbook_id=None,
-                 view_id=None,
-                 connection_id=None,
-                 add_tags=False,
-                 delete_tag=False,
-                 tag_name=None,
-                 create_extracts=False,
-                 delete_extracts=False,
-                 encryption_flag=False,
-                 revision_number=None,
-                 downgrade_target_version=None,
-                 query_views=False,
-                 query_connections=False,
-                 query_workbook_preview_img=False,
-                 query_workbook_view_preview_img=False,
-                 get_workbook_revisions=False,
-                 remove_workbook_revision=False,
-                 download_workbook=False,
-                 download_workbook_pdf=False,
-                 download_workbook_revision=False,
-                 refresh_workbook=False,
-                 get_workbook_downgrade_info=False,
-                 parameter_dict=None):
-        """
-        Builds API endpoints for REST API workbook methods.
-        :param class ts_connection: the Tableau Server connection object
-        :param bool query_workbooks: True if querying all workbooks; False otherwise
-        :param bool query_workbook: True if querying a specific workbook, False otherwise
-        :param bool update_workbook_connection: True if updating a specific workbook connection, False otherwise
-        :param bool publish_workbook: True if publishing a specific workbook, False otherwise
-        :param bool update_workbook: True if updating a specific workbook, False otherwise
-        :param bool delete_workbook: True if deleting a specific workbook, False otherwise
-        :param str workbook_id: the workbook ID
-        :param str view_id: the view ID
-        :param str connection_id: the workbook connection ID
-        :param bool add_tags: True if adding tags, False otherwise
-        :param bool delete_tag: True if deleting a specific tag, False otherwise
-        :param str tag_name: the name of the tag
-        :param bool create_extracts: True if creating extracts for the workbook, False otherwise
-        :param bool delete_extracts: True if deleting extracts for the workbook, False otherwise
-        :param bool encryption_flag: True if encrypting extracts for the workbook, False otherwise
-        :param str revision_number: the revision number of the workbook revision to download
-        :param str downgrade_target_version: the desired downgrade target version for the workbook
-        :param bool query_views: True if querying all views, False otherwise
-        :param bool query_connections: True if querying all connections, False otherwise
-        :param bool query_workbook_preview_img: True if querying a specific preview image, False otherwise
-        :param bool query_workbook_view_preview_img: True if querying a specific preview image, False otherwise
-        :param bool get_workbook_revisions: True if getting all workbook revisions, False otherwise
-        :param bool remove_workbook_revision: True if removing a workbook revision, False otherwise
-        :param bool download_workbook: True if downloading workbook content, False otherwise
-        :param bool download_workbook_pdf: True if downloading a specific workbook's PDF, False otherwise
-        :param bool download_workbook_revision: True if downloading a specific workbook revision, False otherwise
-        :param bool refresh_workbook: True if refreshing a specific workbook, False otherwise
-        :param bool get_workbook_downgrade_info: True if getting downgrade info for a specific workbook, False otherwise
-        :param dict parameter_dict: dictionary of URL parameters to append. The value in each key-value pair is the
-        literal text that will be appended to the URL endpoint
+                 query_workbooks: Optional[bool] = False,
+                 query_workbook: Optional[bool] = False,
+                 publish_workbook: Optional[bool] = False,
+                 update_workbook: Optional[bool] = False,
+                 delete_workbook: Optional[bool] = False,
+                 update_workbook_connection: Optional[bool] = False,
+                 workbook_id: Optional[str] = None,
+                 view_id: Optional[str] = None,
+                 connection_id: Optional[str] = None,
+                 add_tags: Optional[bool] = False,
+                 delete_tag: Optional[bool] = False,
+                 tag_name: Optional[str] = None,
+                 create_extracts: Optional[bool] = False,
+                 delete_extracts: Optional[bool] = False,
+                 encryption_flag: Optional[bool] = False,
+                 revision_number: Optional[str] = None,
+                 downgrade_target_version: Optional[str] = None,
+                 query_views: Optional[bool] = False,
+                 query_connections: Optional[bool] = False,
+                 query_workbook_preview_img: Optional[bool] = False,
+                 query_workbook_view_preview_img: Optional[bool] = False,
+                 get_workbook_revisions: Optional[bool] = False,
+                 remove_workbook_revision: Optional[bool] = False,
+                 download_workbook: Optional[bool] = False,
+                 download_workbook_pdf: Optional[bool] = False,
+                 download_workbook_pptx: Optional[bool] = False,
+                 download_workbook_revision: Optional[bool] = False,
+                 refresh_workbook: Optional[bool] = False,
+                 get_workbook_downgrade_info: Optional[bool] = False,
+                 parameter_dict: Dict[str, Any] = None):
+        """Builds API endpoints for REST API workbook methods.
+
+        Args:
+            ts_connection: The Tableau Server connection object.
+            query_workbooks: True if querying all workbooks; False otherwise.
+            query_workbook: True if querying a specific workbook, False otherwise.
+            update_workbook_connection: True if updating a specific workbook connection, False otherwise.
+            publish_workbook: True if publishing a specific workbook, False otherwise.
+            update_workbook: True if updating a specific workbook, False otherwise.
+            delete_workbook: True if deleting a specific workbook, False otherwise.
+            workbook_id: The ID (luid) for the workbook.
+            view_id: The ID (luid) for the view.
+            connection_id: The ID for the workbook connection.
+            add_tags: True if adding tags, False otherwise.
+            delete_tag: True if deleting a specific tag, False otherwise.
+            tag_name: The name of the tag.
+            create_extracts: True if creating extracts for the workbook, False otherwise.
+            delete_extracts: True if deleting extracts for the workbook, False otherwise.
+            encryption_flag: True if encrypting extracts for the workbook, False otherwise.
+            revision_number: The revision number of the workbook revision to download.
+            downgrade_target_version: The desired downgrade target version for the workbook.
+            query_views: True if querying all views, False otherwise.
+            query_connections: True if querying all connections, False otherwise.
+            query_workbook_preview_img: True if querying a specific preview image, False otherwise.
+            query_workbook_view_preview_img: True if querying a specific preview image, False otherwise.
+            get_workbook_revisions: True if getting all workbook revisions, False otherwise.
+            remove_workbook_revision: True if removing a workbook revision, False otherwise.
+            download_workbook: True if downloading workbook content, False otherwise.
+            download_workbook_pdf: True if downloading a specific workbook as a PDF file, False otherwise.
+            download_workbook_pptx: True if downloading a specific workbook as a Powerpoint file, False otherwise.
+            download_workbook_revision: True if downloading a specific workbook revision, False otherwise.
+            refresh_workbook: True if refreshing a specific workbook, False otherwise.
+            get_workbook_downgrade_info: True if getting downgrade info for a specific workbook, False otherwise.
+            parameter_dict: A Python dict whose values define additional URL parameters. The value in each key-value
+            pair is the literal text that will be appended to the URL endpoint.
         """
 
         super().__init__(ts_connection)
@@ -94,6 +99,7 @@ class WorkbookEndpoint(BaseEndpoint):
         self._remove_workbook_revision = remove_workbook_revision
         self._download_workbook = download_workbook
         self._download_workbook_pdf = download_workbook_pdf
+        self._download_workbook_pptx = download_workbook_pptx
         self._download_workbook_revision = download_workbook_revision
         self._refresh_workbook = refresh_workbook
         self._get_workbook_downgrade_info = get_workbook_downgrade_info
@@ -102,7 +108,7 @@ class WorkbookEndpoint(BaseEndpoint):
         self._validate_inputs()
 
     @property
-    def mutually_exclusive_params(self):
+    def mutually_exclusive_params(self) -> List[Union[bool, str]]:
         return [
             self._query_workbooks,
             self._query_workbook,
@@ -122,50 +128,51 @@ class WorkbookEndpoint(BaseEndpoint):
             self._remove_workbook_revision,
             self._download_workbook,
             self._download_workbook_pdf,
+            self._download_workbook_pptx,
             self._download_workbook_revision,
             self._refresh_workbook,
             self._get_workbook_downgrade_info
         ]
 
-    def _validate_inputs(self):
+    def _validate_inputs(self) -> None:
         valid = True
         if sum(self.mutually_exclusive_params) > 1:
             valid = False
         if not valid:
             self._invalid_parameter_exception()
 
-    def _modify_parameter_dict(self):
+    def _modify_parameter_dict(self) -> None:
         if self._encryption_flag and not self._parameter_dict:
             self._parameter_dict = {}
         if self._encryption_flag:
             self._parameter_dict.update({"encryption_flag": f"encrypt={self._encryption_flag}"})
 
     @property
-    def base_workbook_url(self):
+    def base_workbook_url(self) -> str:
         return "{0}/api/{1}/sites/{2}/workbooks".format(self._connection.server,
                                                         self._connection.api_version,
                                                         self._connection.site_id)
 
     @property
-    def base_workbook_id_url(self):
+    def base_workbook_id_url(self) -> str:
         return "{0}/{1}".format(self.base_workbook_url,
                                 self._workbook_id)
 
     @property
-    def base_workbook_tags_url(self):
+    def base_workbook_tags_url(self) -> str:
         return "{0}/tags".format(self.base_workbook_id_url)
 
     @property
-    def base_delete_workbook_tag_url(self):
+    def base_delete_workbook_tag_url(self) -> str:
         return "{0}/{1}".format(self.base_workbook_tags_url,
                                 self._tag_name)
 
     @property
-    def base_create_extract_url(self):
+    def base_create_extract_url(self) -> str:
         return "{0}/createExtract".format(self.base_workbook_id_url)
 
     @property
-    def base_delete_extract_url(self):
+    def base_delete_extract_url(self) -> str:
         return "{0}/deleteExtract".format(self.base_workbook_id_url)
 
     @property
@@ -173,63 +180,63 @@ class WorkbookEndpoint(BaseEndpoint):
         return "{0}/views".format(self.base_workbook_id_url)
 
     @property
-    def base_workbook_connections_url(self):
+    def base_workbook_connections_url(self) -> str:
         return "{0}/connections".format(self.base_workbook_id_url)
 
     @property
-    def base_workbook_connection_id_url(self):
+    def base_workbook_connection_id_url(self) -> str:
         return "{0}/{1}".format(self.base_workbook_connections_url,
                                 self._connection_id)
 
     @property
-    def base_workbook_preview_url(self):
+    def base_workbook_preview_url(self) -> str:
         return "{0}/previewImage".format(self.base_workbook_id_url)
 
     @property
-    def base_workbook_view_preview_url(self):
+    def base_workbook_view_preview_url(self) -> str:
         return "{0}/{1}/previewImage".format(self.base_workbook_views_url,
                                              self._view_id)
 
     @property
-    def base_workbook_revisions_url(self):
+    def base_workbook_revisions_url(self) -> str:
         return "{0}/revisions".format(self.base_workbook_id_url)
 
     @property
-    def base_workbook_downgrade_version_url(self):
+    def base_workbook_downgrade_version_url(self) -> str:
         return "{0}/downGradeInfo?productVersion={1}".format(self.base_workbook_id_url,
                                                              self._downgrade_target_version)
 
     @property
-    def base_workbook_revision_removal_url(self):
+    def base_workbook_revision_removal_url(self) -> str:
         return "{0}/{1}".format(self.base_workbook_revisions_url,
                                 self._revision_number)
 
     @property
-    def base_workbook_content_url(self):
+    def base_workbook_content_url(self) -> str:
         return "{0}/content".format(self.base_workbook_id_url)
 
     @property
-    def base_workbook_revision_download_url(self):
+    def base_workbook_revision_download_url(self) -> str:
         return "{0}/{1}/content".format(self.base_workbook_revisions_url,
                                         self._revision_number)
 
     @property
-    def base_workbook_download_pdf_url(self):
+    def base_workbook_download_pdf_url(self) -> str:
         return "{0}/pdf".format(self.base_workbook_id_url)
 
     @property
-    def base_workbook_refresh_url(self):
+    def base_workbook_refresh_url(self) -> str:
         return "{0}/refresh".format(self.base_workbook_id_url)
 
     @property
-    def valid_page_orientations(self):
+    def valid_page_orientations(self) -> List[str]:
         return [
             'Portrait',
             'Landscape'
         ]
 
     @property
-    def valid_page_types(self):
+    def valid_page_types(self) -> List[str]:
         return [
             'A3',
             'A4',
@@ -245,7 +252,7 @@ class WorkbookEndpoint(BaseEndpoint):
             'Tabloid'
         ]
 
-    def _validate_parameter_dict(self):
+    def _validate_parameter_dict(self) -> None:
         if self._parameter_dict:
             parameter_keys = [key.lower() for key in self._parameter_dict.keys()]
             if 'orientation' in parameter_keys:
@@ -259,7 +266,7 @@ class WorkbookEndpoint(BaseEndpoint):
                 else:
                     raise self._invalid_parameter_exception()
 
-    def get_endpoint(self):
+    def get_endpoint(self) -> str:
         url = None
         if self._workbook_id:
             if self._query_workbook:
